// Author: Denis Denisov
#include "testlib.h"
#include <sstream>
#include <string>
#include <iostream>
#include <cassert>

using namespace std;

const int MAXN = 100000;
const int MAXM = 500000;
const int MAXC = 1000000000;

class Edge
{
public:
	int be, en;
	int c;
};

int n, m1, m2;
Edge edg[MAXM];
int first[MAXN + 1];
int next[MAXM];
int rfirst[MAXN + 1];
int rnext[MAXM];
int nume;

void AddEdge(int v1, int v2, int c)
{
	edg[nume].be = v1;
	edg[nume].en = v2;
	edg[nume].c = c;
	nume++;
}

void Load()
{
	memset(first, 0xFF, sizeof(first));
	memset(rfirst, 0xFF, sizeof(rfirst));
	nume = 0;
	n = inf.readInt();
	m1 = inf.readInt();
	assert(0 < n && n <= MAXN);
	int i, p, q, c;
	for (i = 0; i < m1; i++)
	{
		p = inf.readInt();
		q = inf.readInt();
		assert(1 <= p && p <= n);
		assert(1 <= q && q <= n);
		AddEdge(p, q, 0);
	}
	m2 = inf.readInt();
	for (i = 0; i < m2; i++)
	{
		p = inf.readInt();
		q = inf.readInt();
		c = inf.readInt();
		assert(1 <= p && p <= n);
		assert(1 <= q && q <= n);
		assert(-MAXC <= c && c <= MAXC);
		AddEdge(p, q, c);
	}
}

int havetaken[MAXM];

int ind[MAXN + 1];
int ctop;
int was[MAXN + 1];

void DfsTop(int ver)
{
	was[ver] = 1;
	int i;
	for (i = first[ver]; i != -1; i = next[i])
	{
		if (was[edg[i].en] != 0) continue;
		DfsTop(edg[i].en);
	}
	ind[ctop++] = ver;
}

int nc[MAXN + 1];
int ccom;

void DfsCom(int ver)
{
	nc[ver] = ccom;
	int i;
	for (i = rfirst[ver]; i != -1; i = rnext[i])
	{
		if (nc[edg[i].be] == 0)
		{
			DfsCom(edg[i].be);
		}
	}
}

bool CSC()
{
	int i;
	//cerr << "We have edges:\n";
	for (i = 0; i < m1 + m2; i++)
	{
		if (i < m1 || havetaken[i])
		{
			next[i] = first[edg[i].be];
			first[edg[i].be] = i;
			rnext[i] = rfirst[edg[i].en];
			rfirst[edg[i].en] = i;
			//cerr << edg[i].be << " " << edg[i].en << "\n";
		}
	}
	memset(was, 0, sizeof(was));
	ctop = 0;
	for (i = 1; i <= n; i++)
	{
		if (was[i] == 0) 
		{
			DfsTop(i);
		}
	}
	memset(nc, 0, sizeof(nc));
	ccom = 0;
	for (i = ctop - 1; i >= 0; i--)
	{
		if (nc[ind[i]] == 0)
		{
			ccom++;
			DfsCom(ind[i]);
		}
	}
	if (ccom > 1) return false;
	return true;
}

int main(int argc, char * argv[])
{
    setName("checker for problem augmentation");
    registerTestlibCmd(argc, argv);
	Load();
	string jans, cans;
	jans = ans.readLine();
	cans = ouf.readLine();
	if ((cans != "YES") && (cans != "NO")) quitf(_wa, "YES or NO expected, %s found.", cans.c_str());
	if (cans == "NO")
	{
		if (jans == "YES") quitf(_wa, "Contestant does not know solution, when jury does.");
		else
		{
			quitf(_ok, "No solution. n = %d m1 = %d m2 = %d", n, m1, m2);
		}
	}
	else
	{
		memset(havetaken, 0, sizeof(havetaken));
		long long jcost, ccost;
		if (jans == "NO") jcost = 1000000000000000000ll;
		else jcost = ans.readLong();
		ccost = ouf.readLong();
		if (ccost > jcost)
		{
			ostringstream oss;
			oss << "Contestant can reach cost " << ccost << ", but jury can " << jcost << ".";
			quitf(_wa, oss.str().c_str());
		}
		int cedges = ouf.readLong();
		if (cedges < 0) quitf(_wa, "We can not add negative number of edges, can we?");
		int i;
		long long contans = 0;
		for (i = 0; i < cedges; i++)
		{
			int cure = ouf.readLong();
			if (cure < 1 || cure > m2) quitf(_wa, "There is no edge with number %d.", cure);
			if (havetaken[cure + m1 - 1] == 1) quitf(_wa, "Edge number %d is already added.", cure);
			havetaken[cure + m1 - 1] = 1;
			contans += edg[cure + m1 - 1].c;
		}
		if (!CSC()) quitf(_wa, "Graph is not strongly connected after adding edges.");
		if (contans != ccost) 
		{
			ostringstream oss;
			oss << "Contestant declared total cost as " << ccost << " but in fact he has " << contans;
			quitf(_wa, oss.str().c_str());
		}
		if (ccost < jcost) quitf(_fail, "Contestant has better answer than jury.");
		else 
		{
			ostringstream oss;
			oss << "n = " << n << " m1 = " << m1 << " m2 = " << m2 << " total cost = " << ccost;
			quitf(_ok, oss.str().c_str());
		}
	}
	quitf(_fail, "Some strange botva appeared!");
}
